@@ -34,12 +34,13 @@
 #include <stdlib.h>
 #include <stdio.h>
 
-static void meta_window_present_delete_dialog (MetaWindow *window);
+static void meta_window_present_delete_dialog (MetaWindow *window,
+                                               guint32     timestamp);
 
 static void
 delete_ping_reply_func (MetaDisplay *display,
                         Window       xwindow,
-                        Time         timestamp,
+                        guint32      timestamp,
                         void        *user_data)
 {
   meta_topic (META_DEBUG_PING,
@@ -292,7 +293,7 @@ io_from_ping_dialog (GIOChannel   *channel,
 static void
 delete_ping_timeout_func (MetaDisplay *display,
                           Window       xwindow,
-                          Time         timestamp,
+                          guint32      timestamp,
                           void        *user_data)
 {
   MetaWindow *window = user_data;
@@ -311,14 +312,14 @@ delete_ping_timeout_func (MetaDisplay *display,
 
   if (window->dialog_pid >= 0)
     {
-      meta_window_present_delete_dialog (window);
+      meta_window_present_delete_dialog (window, timestamp);
       return;
     }
   
   window_id_str = g_strdup_printf ("0x%lx", window->xwindow);
 
   sprintf (numbuf, "%d", window->screen->number);
-  sprintf (timestampbuf, "%lu", timestamp);
+  sprintf (timestampbuf, "%u", timestamp);
   
   argv[0] = METACITY_LIBEXECDIR"/metacity-dialog";
   argv[1] = "--screen";
@@ -364,7 +365,7 @@ delete_ping_timeout_func (MetaDisplay *display,
 
 void
 meta_window_delete (MetaWindow  *window,
-                    Time         timestamp)
+                    guint32      timestamp)
 {
   meta_error_trap_push (window->display);
   if (window->delete_window)
@@ -476,7 +477,7 @@ meta_window_free_delete_dialog (MetaWindow *window)
 }
 
 static void
-meta_window_present_delete_dialog (MetaWindow *window)
+meta_window_present_delete_dialog (MetaWindow *window, guint32 timestamp)
 {
   meta_topic (META_DEBUG_PING,
               "Presenting existing ping dialog for %s\n",
@@ -501,8 +502,7 @@ meta_window_present_delete_dialog (MetaWindow *window)
               w->res_class &&
               g_strcasecmp (w->res_class, "metacity-dialog") == 0)
             {
-              meta_window_activate (w,
-                                    meta_display_get_current_time (w->display));
+              meta_window_activate (w, timestamp);
               break;
             }
           
