@@ -62,7 +62,7 @@ typedef struct MetaEdgeResistanceData MetaEdgeResistanceData;
 
 typedef void (* MetaWindowPingFunc) (MetaDisplay *display,
 				     Window       xwindow,
-				     Time         timestamp,
+				     guint32      timestamp,
 				     gpointer     user_data);
 
 
@@ -194,10 +194,10 @@ struct _MetaDisplay
   MetaWindow *expected_focus_window;
 
   /* last timestamp passed to XSetInputFocus */
-  Time last_focus_time;
+  guint32 last_focus_time;
 
   /* last user interaction time in any app */
-  Time last_user_time;
+  guint32 last_user_time;
 
   /* whether we're using mousenav (only relevant for sloppy&mouse focus modes;
    * !mouse_mode means "keynav mode")
@@ -226,12 +226,6 @@ struct _MetaDisplay
                               XErrorEvent *error);  
   int server_grab_count;
 
-  /* for double click */
-  Time last_button_time;
-  Window last_button_xwindow;
-  int last_button_num;
-  guint is_double_click : 1;
-
   /* serials of leave/unmap events that may
    * correspond to an enter event we should
    * ignore
@@ -267,7 +261,7 @@ struct _MetaDisplay
   guint       grab_have_pointer : 1;
   guint       grab_have_keyboard : 1;
   guint       grab_wireframe_active : 1;
-  guint       grab_was_cancelled : 1;
+  guint       grab_was_cancelled : 1;    /* Only used in wireframe mode */
   MetaRectangle grab_wireframe_rect;
   MetaRectangle grab_wireframe_last_xor_rect;
   MetaRectangle grab_initial_window_pos;
@@ -275,12 +269,13 @@ struct _MetaDisplay
   gboolean    grab_threshold_movement_reached; /* raise_on_click == FALSE.    */
   MetaResizePopup *grab_resize_popup;
   GTimeVal    grab_last_moveresize_time;
-  Time        grab_motion_notify_time;
+  guint32     grab_motion_notify_time;
   int         grab_wireframe_last_display_width;
   int         grab_wireframe_last_display_height;
   GList*      grab_old_window_stacking;
   MetaEdgeResistanceData *grab_edge_resistance_data;
-  
+  unsigned int grab_last_user_action_was_snap;
+
   /* we use property updates as sentinels for certain window focus events
    * to avoid some race conditions on EnterNotify events
    */
@@ -288,7 +283,7 @@ struct _MetaDisplay
 
 #ifdef HAVE_XKB
   int         xkb_base_event_type;
-  Time        last_bell_time;
+  guint32     last_bell_time;
 #endif
 #ifdef HAVE_XSYNC
   /* alarm monitoring client's _NET_WM_SYNC_REQUEST_COUNTER */
@@ -301,8 +296,8 @@ struct _MetaDisplay
   int             n_screen_bindings;
   MetaKeyBinding *window_bindings;
   int             n_window_bindings;
-  unsigned int min_keycode;
-  unsigned int max_keycode;
+  int             min_keycode;
+  int             max_keycode;
   KeySym *keymap;
   int keysyms_per_keycode;
   XModifierKeymap *modmap;
@@ -351,7 +346,6 @@ struct _MetaDisplay
   int render_error_base;
 #endif
 #ifdef HAVE_XSYNC
-  unsigned int grab_last_user_action_was_snap;
   unsigned int have_xsync : 1;
 #define META_DISPLAY_HAS_XSYNC(display) ((display)->have_xsync)
 #else
@@ -393,8 +387,9 @@ struct _MetaDisplay
      (time2) != 0)                                                      \
   )
 
-gboolean      meta_display_open                (const char  *name);
-void          meta_display_close               (MetaDisplay *display);
+gboolean      meta_display_open                (void);
+void          meta_display_close               (MetaDisplay *display,
+                                                guint32      timestamp);
 MetaScreen*   meta_display_screen_for_root     (MetaDisplay *display,
                                                 Window       xroot);
 MetaScreen*   meta_display_screen_for_x_screen (MetaDisplay *display,
@@ -403,13 +398,14 @@ MetaScreen*   meta_display_screen_for_xwindow  (MetaDisplay *display,
                                                 Window       xindow);
 void          meta_display_grab                (MetaDisplay *display);
 void          meta_display_ungrab              (MetaDisplay *display);
-gboolean      meta_display_is_double_click     (MetaDisplay *display);
 
 void          meta_display_unmanage_screen     (MetaDisplay *display,
-                                                MetaScreen  *screen);
+                                                MetaScreen  *screen,
+                                                guint32      timestamp);
 
 void          meta_display_unmanage_windows_for_screen (MetaDisplay *display,
-                                                        MetaScreen  *screen);
+                                                        MetaScreen  *screen,
+                                                        guint32      timestamp);
 
 /* A given MetaWindow may have various X windows that "belong"
  * to it, such as the frame window.
@@ -438,7 +434,7 @@ void     meta_display_set_grab_op_cursor (MetaDisplay *display,
                                           MetaGrabOp   op,
                                           gboolean     change_pointer,
                                           Window       grab_xwindow,
-                                          Time         timestamp);
+                                          guint32      timestamp);
 
 gboolean meta_display_begin_grab_op (MetaDisplay *display,
                                      MetaScreen  *screen,
@@ -448,11 +444,11 @@ gboolean meta_display_begin_grab_op (MetaDisplay *display,
                                      int          event_serial,
                                      int          button,
                                      gulong       modmask,
-                                     Time         timestamp,
+                                     guint32      timestamp,
                                      int          root_x,
                                      int          root_y);
 void     meta_display_end_grab_op   (MetaDisplay *display,
-                                     Time         timestamp);
+                                     guint32      timestamp);
 
 void    meta_display_check_threshold_reached (MetaDisplay *display,
                                               int          x,
@@ -489,19 +485,20 @@ void meta_display_retheme_all (void);
 void meta_display_set_cursor_theme (const char *theme, 
 				    int         size);
 
-void     meta_display_ping_window              (MetaDisplay        *display,
-						MetaWindow         *window,
-						Time                timestamp,
-						MetaWindowPingFunc  ping_reply_func,
-						MetaWindowPingFunc  ping_timeout_func,
-						void               *user_data);
+void meta_display_ping_window      (MetaDisplay        *display,
+                                    MetaWindow         *window,
+                                    guint32             timestamp,
+                                    MetaWindowPingFunc  ping_reply_func,
+                                    MetaWindowPingFunc  ping_timeout_func,
+                                    void               *user_data);
 gboolean meta_display_window_has_pending_pings (MetaDisplay        *display,
 						MetaWindow         *window);
 
 typedef enum
 {
   META_TAB_LIST_NORMAL,
-  META_TAB_LIST_DOCKS
+  META_TAB_LIST_DOCKS,
+  META_TAB_LIST_GROUP
 } MetaTabList;
 
 typedef enum
@@ -552,7 +549,7 @@ gboolean meta_display_focus_sentinel_clear (MetaDisplay *display);
 void meta_display_set_input_focus_window   (MetaDisplay *display, 
                                             MetaWindow  *window,
                                             gboolean     focus_frame,
-                                            Time         timestamp);
+                                            guint32      timestamp);
 
 /* meta_display_focus_the_no_focus_window is called when the
  * designated no_focus_window should be focused, but is otherwise the
@@ -560,7 +557,7 @@ void meta_display_set_input_focus_window   (MetaDisplay *display,
  */
 void meta_display_focus_the_no_focus_window (MetaDisplay *display, 
                                              MetaScreen  *screen,
-                                             Time         timestamp);
+                                             guint32      timestamp);
 
 void meta_display_queue_autoraise_callback  (MetaDisplay *display,
                                              MetaWindow  *window);
