@@ -23,6 +23,7 @@
 #include "effects.h"
 #include "display.h"
 #include "ui.h"
+#include "window.h"
 
 #ifdef HAVE_SHAPE
 #include <X11/extensions/shape.h>
@@ -72,6 +73,132 @@ typedef struct
   
 } BoxAnimationContext;
 
+struct MetaEffectPriv
+{
+  MetaEffectFinished finished;
+  gpointer	     finished_data;
+};
+
+static void run_default_effect_handler (MetaEffect *effect);
+static void run_handler (MetaEffect *effect);
+
+static MetaEffectHandler effect_handler;
+static gpointer		 effect_handler_data;
+
+void
+meta_push_effect_handler (MetaEffectHandler   handler,
+			  gpointer            data)
+{
+    effect_handler = handler;
+    effect_handler_data = data;
+}
+
+void
+meta_pop_effect_handler (void)
+{
+    /* FIXME: not implemented yet */
+    g_assert_not_reached ();
+}
+
+static MetaEffect *
+create_effect (MetaEffectType      type,
+	       MetaWindow         *window,
+	       MetaEffectFinished  finished,
+	       gpointer		   finished_data)
+{
+    MetaEffect *effect = g_new (MetaEffect, 1);
+
+    effect->type = type;
+    effect->window = window;
+    effect->priv = g_new (MetaEffectPriv, 1);
+    effect->priv->finished = finished;
+    effect->priv->finished_data = finished_data;
+
+    return effect;
+}
+
+void
+meta_effect_end (MetaEffect         *effect)
+{
+    if (effect->priv->finished)
+	effect->priv->finished (effect, effect->priv->finished_data);
+    
+    g_free (effect->priv);
+    g_free (effect);
+}
+
+void
+meta_effect_run_focus (MetaWindow	    *window,
+		       MetaEffectFinished   finished,
+		       gpointer		    data)
+{
+    MetaEffect *effect;
+
+    g_return_if_fail (window != NULL);
+
+    effect = create_effect (META_EFFECT_FOCUS, window, finished, data);
+    
+    run_handler (effect);
+}
+
+void
+meta_effect_run_minimize (MetaWindow         *window,
+			  MetaRectangle      *window_rect,
+			  MetaRectangle	     *icon_rect,
+			  MetaEffectFinished  finished,
+			  gpointer            data)
+{
+    MetaEffect *effect;
+
+    g_return_if_fail (window != NULL);
+    g_return_if_fail (icon_rect != NULL);
+    
+    effect = create_effect (META_EFFECT_MINIMIZE, window, finished, data);
+
+    effect->u.minimize.window_rect = *window_rect;
+    effect->u.minimize.icon_rect = *icon_rect;
+
+    run_handler (effect);
+}
+
+void
+meta_effect_run_unminimize (MetaWindow         *window,
+			    MetaRectangle      *window_rect,
+			    MetaRectangle      *icon_rect,
+			    MetaEffectFinished  finished,
+			    gpointer            data)
+{
+    MetaEffect *effect;
+
+    g_return_if_fail (window != NULL);
+    g_return_if_fail (icon_rect != NULL);
+    
+    effect = create_effect (META_EFFECT_UNMINIMIZE, window, finished, data);
+
+    effect->u.minimize.window_rect = *window_rect;
+    effect->u.minimize.icon_rect = *icon_rect;
+
+    run_handler (effect);
+}
+
+void
+meta_effect_run_close (MetaWindow         *window,
+		       MetaEffectFinished  finished,
+		       gpointer            data)
+{
+    MetaEffect *effect;
+    
+    g_return_if_fail (window != NULL);
+
+    effect = create_effect (META_EFFECT_CLOSE, window,
+			    finished, data);
+
+    run_handler (effect);
+}
+
+
+/* old ugly minimization effect */
+
 static void
 update_wireframe_window (MetaDisplay         *display,
                          Window               xwindow,
@@ -374,7 +501,7 @@ meta_effects_draw_box_animation (MetaScreen     *screen,
       attrs.override_redirect = True;
       attrs.background_pixel = BlackPixel (screen->display->xdisplay,
                                            screen->number);
-      
+
       context->wireframe_xwindow = XCreateWindow (screen->display->xdisplay,
                                                   screen->xroot,
                                                   initial_rect->x,
@@ -455,19 +582,31 @@ draw_xor_rect (MetaScreen          *screen,
    * a bit oddly.
    */
   XSegment segments[8];
+  MetaRectangle shrunk_rect;
   int i;
   
 #define LINE_WIDTH META_WIREFRAME_XOR_LINE_WIDTH
-  
+
+  /* We don't want the wireframe going outside the window area.
+   * It makes it harder for the user to position windows and it exposes other
+   * annoying bugs.
+   */
+  shrunk_rect = *rect;
+
+  shrunk_rect.x += LINE_WIDTH / 2 + LINE_WIDTH % 2;
+  shrunk_rect.y += LINE_WIDTH / 2 + LINE_WIDTH % 2;
+  shrunk_rect.width -= LINE_WIDTH + 2 * (LINE_WIDTH % 2);
+  shrunk_rect.height -= LINE_WIDTH + 2 * (LINE_WIDTH % 2);
+
   XDrawRectangle (screen->display->xdisplay,
                   screen->xroot,
                   screen->root_xor_gc,
-                  rect->x, rect->y,
-                  rect->width, rect->height);
+                  shrunk_rect.x, shrunk_rect.y,
+                  shrunk_rect.width, shrunk_rect.height);
 
   /* Don't put lines inside small rectangles where they won't fit */
-  if (rect->width < (LINE_WIDTH * 4) ||
-      rect->height < (LINE_WIDTH * 4))
+  if (shrunk_rect.width < (LINE_WIDTH * 4) ||
+      shrunk_rect.height < (LINE_WIDTH * 4))
     return;
 
   if ((width >= 0) && (height >= 0))
@@ -502,11 +641,11 @@ draw_xor_rect (MetaScreen          *screen,
               box_height = text_height + 2 * LINE_WIDTH;
 
 
-              box_x = rect->x + (rect->width - box_width) / 2;
-              box_y = rect->y + (rect->height - box_height) / 2;
+              box_x = shrunk_rect.x + (shrunk_rect.width - box_width) / 2;
+              box_y = shrunk_rect.y + (shrunk_rect.height - box_height) / 2;
 
-              if ((box_width < rect->width) &&
-                  (box_height < rect->height))
+              if ((box_width < shrunk_rect.width) &&
+                  (box_height < shrunk_rect.height))
                 {
                   XFillRectangle (screen->display->xdisplay,
                                   screen->xroot,
@@ -525,32 +664,32 @@ draw_xor_rect (MetaScreen          *screen,
 
               XFreeFontInfo (NULL, font_struct, 1);
 
-              if ((box_width + LINE_WIDTH) >= (rect->width / 3))
+              if ((box_width + LINE_WIDTH) >= (shrunk_rect.width / 3))
                 return;
 
-              if ((box_height + LINE_WIDTH) >= (rect->height / 3))
+              if ((box_height + LINE_WIDTH) >= (shrunk_rect.height / 3))
                 return;
             }
         }
     }
 
   /* Two vertical lines at 1/3 and 2/3 */
-  segments[0].x1 = rect->x + rect->width / 3;
-  segments[0].y1 = rect->y + LINE_WIDTH / 2 + LINE_WIDTH % 2;
+  segments[0].x1 = shrunk_rect.x + shrunk_rect.width / 3;
+  segments[0].y1 = shrunk_rect.y + LINE_WIDTH / 2 + LINE_WIDTH % 2;
   segments[0].x2 = segments[0].x1;
-  segments[0].y2 = rect->y + rect->height - LINE_WIDTH / 2;  
+  segments[0].y2 = shrunk_rect.y + shrunk_rect.height - LINE_WIDTH / 2;  
 
   segments[1] = segments[0];
-  segments[1].x1 = rect->x + (rect->width / 3) * 2;
+  segments[1].x1 = shrunk_rect.x + (shrunk_rect.width / 3) * 2;
   segments[1].x2 = segments[1].x1;
 
   /* Now make two horizontal lines at 1/3 and 2/3, but not
    * overlapping the verticals
    */
 
-  segments[2].x1 = rect->x + LINE_WIDTH / 2 + LINE_WIDTH % 2;
+  segments[2].x1 = shrunk_rect.x + LINE_WIDTH / 2 + LINE_WIDTH % 2;
   segments[2].x2 = segments[0].x1 - LINE_WIDTH / 2;
-  segments[2].y1 = rect->y + rect->height / 3;
+  segments[2].y1 = shrunk_rect.y + shrunk_rect.height / 3;
   segments[2].y2 = segments[2].y1;
 
   segments[3] = segments[2];
@@ -559,7 +698,7 @@ draw_xor_rect (MetaScreen          *screen,
   
   segments[4] = segments[3];
   segments[4].x1 = segments[3].x2 + LINE_WIDTH;
-  segments[4].x2 = rect->x + rect->width - LINE_WIDTH / 2;
+  segments[4].x2 = shrunk_rect.x + shrunk_rect.width - LINE_WIDTH / 2;
 
   /* Second horizontal line is just like the first, but
    * shifted down
@@ -568,7 +707,7 @@ draw_xor_rect (MetaScreen          *screen,
   while (i < 8)
     {
       segments[i] = segments[i - 3];
-      segments[i].y1 = rect->y + (rect->height / 3) * 2;
+      segments[i].y1 = shrunk_rect.y + (shrunk_rect.height / 3) * 2;
       segments[i].y2 = segments[i].y1;
       ++i;
     }
@@ -612,3 +751,34 @@ meta_effects_end_wireframe (MetaScreen          *screen,
   meta_ui_pop_delay_exposes (screen->ui);
 }
 
+static void
+run_default_effect_handler (MetaEffect *effect)
+{
+    switch (effect->type)
+    {
+    case META_EFFECT_MINIMIZE:
+	meta_effects_draw_box_animation (effect->window->screen,
+					 &(effect->u.minimize.window_rect),
+					 &(effect->u.minimize.icon_rect),
+					 META_MINIMIZE_ANIMATION_LENGTH,
+					 META_BOX_ANIM_SCALE);
+	break;
+
+    default:
+	break;
+    }
+}
+
+static void
+run_handler (MetaEffect *effect)
+{
+    if (effect_handler)
+    {
+	effect_handler (effect, effect_handler_data);
+    }
+    else
+    {
+	run_default_effect_handler (effect);
+	meta_effect_end (effect);
+    }
+}
